﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace RSPhotography.DataAccess.DataAccessors
{
    public abstract class BaseAccessor : IDisposable
    {
        #region Constructor

        public BaseAccessor()
        {
            _context = new RSPhotographyEntities();
            _context.Configuration.LazyLoadingEnabled = false;
            _context.Configuration.ProxyCreationEnabled = false;
            //true is the default, but I've added an explicit assignment anyway
            _context.Configuration.AutoDetectChangesEnabled = true; 
        }

        #endregion

        #region Private Members

        private RSPhotographyEntities _context;
        private DbContextTransaction _transaction = null;

        #endregion

        #region Internal Properties

        protected DbContextTransaction Transaction
        {
            get { return _transaction; }
            set { _transaction = value; }
        }

        /// <summary>
        /// Provides access to the DbContext
        /// </summary>
        internal RSPhotographyEntities DataContext
        {
            get { return _context; }
            set { _context = value; }
        }

        #endregion

        #region Public Functions

        /// <summary>
        /// Create a new Tx.  If there is an active Tx, it is disposed of (not committed).
        /// </summary>
        public void BeginTransaction()
        {
            if (_transaction != null)
            {
                _transaction.Dispose();
            }
            _transaction = DataContext.Database.BeginTransaction();
        }

        /// <summary>
        /// Commit current Tx
        /// </summary>
        public void CommitTransaction()
        {
            if (_transaction != null)
            {
                _transaction.Commit();
            }
        }

        /// <summary>
        /// Rolls back current Tx
        /// </summary>
        public void RollbackTransaction()
        {
            if (_transaction != null)
            {
                _transaction.Rollback();
            }
        }

        #endregion

        #region Implements IDispose

        /// <summary>
        /// Ensure the Data Context is cleaned up
        /// </summary>
        public void Dispose()
        {
            if (_context != null)
            {
                _context.Dispose();
                _context = null;
            }
        }

        #endregion        
    }
}
