﻿
#region Using Directives
using RSPhotography.DataAccess.HelperClasses;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Data.Entity.Migrations;
using System.Linq.Expressions;
#endregion

namespace RSPhotography.DataAccess.DataAccessors
{
    /// <summary>
    /// Primary Data Access interface for Catalog entities
    /// </summary>
    public class CatalogDataAccessor : DataAccessor<Catalog>
    {
        #region Constructors

        public CatalogDataAccessor() : base() { }

        public CatalogDataAccessor(BaseAccessor existing) : base(existing.DataContext) { }

        internal CatalogDataAccessor(RSPhotographyEntities context) : base(context) { }      

        #endregion

        #region Public Functions

        public ReadOnlyCollection<Catalog> GetAllItems()
        {
            return DataContext.Catalogs.ToList().AsReadOnly();
        }

        /// <summary>
        /// Attempt to filter Catalog Items by Location
        /// </summary>
        /// <param name="loc"></param>
        /// <returns></returns>
        public ReadOnlyCollection<Catalog> FindItemsByLocation(Location loc)
        {
            if (loc.LocationId <= 0 || loc.RegionId <= 0)
            {
                throw new ArgumentException("The Location's LocationId or RegionId must be specified");
            }

            IQueryable<Catalog> query = (from a in DataContext.Catalogs
                                            .Include("Locations")                                           
                                        select a);
            if (loc.LocationId > 0)
            {
                query = query.Where(x => x.Locations.Any(y => y.LocationId == loc.LocationId));
            }
            if (loc.RegionId > 0)
            {
                query = query.Where(x => x.Locations.Any(y => y.RegionId == loc.RegionId));
            }

            return query.ToList().AsReadOnly();
        }       

        #endregion       
    }
}
