﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace RSPhotography.DataAccess.DataAccessors
{
    public interface IDataAccessor<T> where T : EntityBase
    {
        /// <summary>
        /// Returns a Queryable interface
        /// </summary>
        /// <returns></returns>
        IQueryable<T> Select();

        /// <summary>
        /// 
        /// </summary>
        /// <param name="filter"></param>
        /// <param name="navigationProperties"></param>
        /// <returns></returns>
        IList<T> GetEntities(Func<T, bool> filter, params Expression<Func<T, object>>[] navigationProperties);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="filter"></param>
        /// <param name="navigationProperties"></param>
        /// <returns></returns>
        T GetEntity(Func<T, bool> filter, params Expression<Func<T, object>>[] navigationProperties);
       
        /// <summary>
        /// Provides a function for inserting or updating entities
        /// </summary>
        /// <param name="entities"></param>
        void InsertOrUpdate(params T[] entities);

        /// <summary>
        /// Provides a function for inserting or updating entities
        /// </summary>
        /// <param name="entities"></param>
        void InsertOrUpdate<T2>(params T2[] entities) where T2 : EntityBase;

        /// <summary>
        /// 
        /// </summary>
        /// <typeparam name="T2"></typeparam>
        /// <param name="parent"></param>
        /// <param name="expression"></param>
        /// <param name="state"></param>
        /// <param name="children"></param>
        void ModifyRelatedEntities<T2>(T parent, Expression<Func<T, object>> collection, EntityState state, params T2[] children) where T2 : EntityBase;

        /// <summary>
        /// 
        /// </summary>
        /// <typeparam name="T2"></typeparam>
        /// <param name="parent"></param>
        /// <param name="collectionName"></param>
        /// <param name="state"></param>
        /// <param name="children"></param>
        void ModifyRelatedEntities<T2>(T parent, string collectionName, EntityState state, params T2[] children) where T2 : EntityBase;    

        /// <summary>
        /// Removes an entity or entities
        /// </summary>
        /// <param name="entities"></param>
        void Delete(params T[] entities);

        /// <summary>
        /// Removes an entity or entities
        /// </summary>
        /// <param name="entities"></param>
        void Delete<T2>(params T2[] entities) where T2 : EntityBase;

        /// <summary>
        /// Commits pending data changes
        /// </summary>
        void SaveChanges();
    }
}
