﻿using System;
using System.Text;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using RSPhotography.DataAccess.DataAccessors;
using RSPhotography.DataAccess;
using System.Linq;
using System.Data.Entity;
using RSPhotography.DataAccess.Enums;

namespace RSPhotography.UnitTests
{
    /// <summary>
    /// Summary description for InsertDelete
    /// </summary>
    [TestClass]
    public class InsertDelete
    {
        [TestMethod]
        public void TryAddDelete()
        {
            using (SizeDataAccessor a = new SizeDataAccessor())
            {
                Size obj = new Size();
                obj.Height = 0;
                obj.Width = 0;
                obj.SizeId = 100; //out of the standard range
                obj.IsCustom = false;
                obj.Description = "Unit Test";
                obj.Dimensions = DateTime.Now.ToString("yyyy-MM-dd hh:mm:ss");
                obj.State = ObjectState.Added;
                a.InsertOrUpdate(obj);
                a.SaveChanges();
            }
            using (SizeDataAccessor a = new SizeDataAccessor())
            {
                var verify = a.GetEntity(x => x.SizeId == 100);
                Assert.IsNotNull(verify, "Should be saved to the database");

                a.Delete(verify);
                a.SaveChanges();

                verify = a.GetEntity(x => x.SizeId == 100);
                Assert.IsNull(verify, "Should be removed from the database");
            }
        }

        [TestMethod]
        public void TryAddQueryDelete()
        {
            using (SizeDataAccessor a = new SizeDataAccessor())
            {
                Size obj = new Size();
                obj.Height = 0;
                obj.Width = 0;
                obj.SizeId = 100; //out of the standard range
                obj.IsCustom = false;
                obj.Description = "Unit Test";
                obj.Dimensions = DateTime.Now.ToString("yyyy-MM-dd hh:mm:ss");
                obj.State = ObjectState.Added;
                a.InsertOrUpdate(obj);
                a.SaveChanges();
            }
            
            using (SizeDataAccessor a = new SizeDataAccessor())
            {
                Size entry = a.CreateQuery().Where(x => x.SizeId == 100).FirstOrDefault();
                Assert.IsTrue(entry != null, "Should find the new record");
                Assert.IsTrue(entry.SizeId == 100, "Should find the new record by SizeId == 100");
                entry.State = ObjectState.Deleted;
                a.Delete(entry);
                a.SaveChanges();
            }
        }

        private Size CreateItem(int uniqueCount)
        {
            Size obj = new Size();
            obj.Height = 0;
            obj.Width = 0;
            obj.SizeId = 100 + uniqueCount; //out of the standard range
            obj.IsCustom = false;
            obj.Description = "Unit Test";
            obj.Dimensions = String.Format("{0}:[{1}]", DateTime.Now.ToString("MM-dd hh:mm:ss"), uniqueCount);
            obj.State = ObjectState.Added;
            return obj;
        }

        [TestMethod]
        public void TryAddBulk()
        {
            List<Size> testSet = new List<Size>();
            for(int i = 0; i != 5; ++i)
            {
                Size item = CreateItem(i);
                testSet.Add(item);
            }
            using (SizeDataAccessor a = new SizeDataAccessor())
            {
                testSet.AddRange(a.CreateQuery().Take(5).ToList());
                //a.BeginTransaction();
                a.InsertOrUpdate(testSet.ToArray());
                a.SaveChanges();
                //a.RollbackTransaction();
                //a.CommitTransaction();
            }
            using (SizeDataAccessor a = new SizeDataAccessor())
            {
                testSet.RemoveRange(5, 5);
                a.Delete(testSet.ToArray());
                a.SaveChanges();
            }
        }

        [TestMethod]
        public void TryUpdateBulk()
        {
using (CatalogDataAccessor a = new CatalogDataAccessor())
{
    string originalDescription = String.Empty;

    IQueryable<Catalog> query = a.CreateQuery();
    query = query.Take(5);                                
    query = query.Include("Sizes");
    var result = query.ToList();
                
    Assert.IsTrue(result.Count > 2, "Should find at least 2 results");
    Catalog item = result[1];
    Assert.IsTrue(item.Sizes.Count > 0, "Should be at least one size");
    Size editItem = item.Sizes.First();
                
    originalDescription = editItem.Description;
    editItem.Description = "Updated By Unit Test";
    editItem.State = ObjectState.Modified;

    a.InsertOrUpdate(result.ToArray());
    a.SaveChanges();

    editItem.Description = originalDescription;

    a.InsertOrUpdate(result.ToArray());
    a.SaveChanges();
}
        }

    }
}
