﻿using System;
using System.Text;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using RSPhotography.DataAccess;
using System.Data.Entity;
using System.Linq;
using System.Diagnostics;
using RSPhotography.DataAccess.DataAccessors;

namespace RSPhotography.UnitTests
{
    /// <summary>
    /// Summary description for QueryTests
    /// </summary>
    [TestClass]
    public class QueryTests
    {

        [TestMethod]
        public void SingleEntityQuery()
        {
            using(CatalogDataAccessor access = new CatalogDataAccessor())
            {
                var result = access.GetEntity(x => x.CatalogId == 1);
                Assert.IsNotNull(result);
            }
        }

        [TestMethod]
        public void SingleEntityQueryWithNavigation()
        {
            using (CatalogDataAccessor access = new CatalogDataAccessor())
            {
                var result = access.GetEntity(x => x.CatalogId == 1, x => x.Files);
                Assert.IsNotNull(result);
                Assert.IsTrue(result.Files.Count > 0);
            }
        }

        [TestMethod]
        public void MultipleEntityQuery()
        {
            using (CatalogDataAccessor access = new CatalogDataAccessor())
            {  
                var result = access.GetEntities(x => x.CatalogId < 5);
                Assert.IsNotNull(result);                  
            }
        }

        [TestMethod]
        public void MultipleEntityQueryWithNavigation()
        {
            using (CatalogDataAccessor access = new CatalogDataAccessor())
            {
                var result = access.GetEntities(x => x.CatalogId < 5, x => x.Files);
                Assert.IsNotNull(result);
                Assert.IsTrue(result[0].Files.Count > 0);
            }
        }

        [TestMethod]
        public void CreateQuery()
        {
            using (CatalogDataAccessor a = new CatalogDataAccessor())
            {
                IQueryable<Catalog> query = a.CreateQuery();
                
                query = query.Where(x => x.CatalogId < 10);
                query = query.Where(x => x.Title.Contains("s"));
                
                var result = query.ToList();

                Assert.IsTrue(result.Count > 0, "Should be at least one match");

                foreach (Catalog s in result)
                {
                    Trace.WriteLine(s.Title);
                }
            }
        }

        [TestMethod]
        public void CreateQueryStructured()
        {
            using (CatalogDataAccessor a = new CatalogDataAccessor())
            {
                IQueryable<Catalog> query = a.CreateQuery();

                query = query.Where(x => x.CatalogId < 10);
                query = query.Where(x => x.Title.Contains("s"));
                query = query.Include("Locations");
                query = query.Include("Sizes");

                var result = query.ToList();

                Assert.IsTrue(result.Count > 0, "Should be at least one match");

                foreach (Catalog s in result)
                {
                    Trace.WriteLine(s.Title);
                }
            }
        }
    }
}
