﻿using System;
using System.Text;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using RSPhotography.WebServices.UnitTests.ServiceReference;

namespace RSPhotography.WebServices.UnitTests
{
    /// <summary>
    /// Summary description for InsertUpdateDelete
    /// </summary>
    [TestClass]
    public class InsertUpdateDelete
    {
        private Size CreateItem(int uniqueCount)
        {
            Size obj = new Size();
            obj.Height = 0;
            obj.Width = 0;
            obj.SizeId = 100 + uniqueCount; //out of the standard range
            obj.IsCustom = false;
            obj.Description = "Unit Test";
            obj.Dimensions = String.Format("{0}:[{1}]", DateTime.Now.ToString("MM-dd hh:mm:ss"), uniqueCount);
            return obj;
        }

        [TestMethod]
        public void WSInsert()
        {
            List<Size> testSet = new List<Size>();

            for (int i = 0; i != 5; ++i)
            {
                Size item = CreateItem(i);
                testSet.Add(item);
            }

            using (CatalogServiceClient c = new CatalogServiceClient())
            {
                c.InsertOrUpdateSize(testSet.ToArray());
            }            
        }

        [TestMethod]
        public void WSInsertUpdate()
        {
            List<Size> testSet = new List<Size>();

            for (int i = 0; i != 5; ++i)
            {
                Size item = CreateItem(i);
                testSet.Add(item);
            }
            
            using (CatalogServiceClient c = new CatalogServiceClient())
            {
                testSet.AddRange(c.GetSizes(5));
                c.InsertOrUpdateSize(testSet.ToArray());              
            }
        }

        [TestMethod]
        public void WSUpdate()
        {
            string originalValue = String.Empty;
            Size updatedEntity = null;

            using (CatalogServiceClient c = new CatalogServiceClient())
            {
                var array = c.GetSizes(5);
                Assert.IsTrue(array.Length > 2, "Should be at least 2 values");
                originalValue = array[1].Description;
                array[1].Description = "Unit Test Update";
                updatedEntity = array[1];

                c.InsertOrUpdateSize(array);
            }

            using (CatalogServiceClient c = new CatalogServiceClient())
            {
                updatedEntity.Description = originalValue;
                c.InsertOrUpdateSizeEntity(updatedEntity);
            }
        }

        [TestMethod]
        public void WSUpdateGenre()
        {
            string originalValue = String.Empty;
            string originalBaseValue = String.Empty;
            Genre result = null;
            Catalog editedItem = null;

            using (CatalogServiceClient c = new CatalogServiceClient())
            {
                result = c.GetItemsByGenre("Landscape");
                Assert.IsNotNull(result, "Should have returned a valid result");
                Assert.IsTrue(result.Catalogs.Length > 1, "Should be at least one child Catalog");
               
                editedItem = result.Catalogs[0];
                editedItem.State = ObjectState.Modified;
                originalValue = editedItem.Title;
                editedItem.Title = "Unit Test Updated";
                originalBaseValue = result.Description;
                result.Description = "New Description";
                result.State = ObjectState.Modified;
                c.UpdateGenre(result);

                result.Description = originalBaseValue;
                result.State = ObjectState.Modified;
                editedItem.Title = originalValue;
                editedItem.State = ObjectState.Modified;
                c.UpdateGenre(result);
            }           
        }

        [TestMethod]
        public void WSUpdateGenres()
        {
            string originalValue = String.Empty;
            Catalog editedItem = null;

            using (CatalogServiceClient c = new CatalogServiceClient())
            {
                Genre[] result = c.GetGenres(2);
                Assert.IsTrue(result.Length > 1, "Should return at least one result");
                Assert.IsTrue(result[0].Catalogs.Length > 0, "Should have at least one Catalog");
                editedItem = result[0].Catalogs[0];

                originalValue = editedItem.Title;
                editedItem.Title = "Unit Test Updated";
                c.UpdateGenres(result);

                editedItem.Title = originalValue;
                c.UpdateGenres(result);
            }
        }
    }
}
