﻿using RSPhotography.DataAccess;
using RSPhotography.DataAccess.DataAccessors;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.Text;
using System.Data.Entity;

namespace RSPhotography.WebServices
{
    public class CatalogService : ICatalogService
    {
        public ReadOnlyCollection<Catalog> GetCatalogByLocation(Location value)
        {
            try
            {
                using (DataAccessor<Catalog> BasicDataAccessor = new DataAccessor<Catalog>())
                {
                    return BasicDataAccessor.CreateQuery().Include("Locations").Where(x => x.Locations.Any(y => y.LocationId == value.LocationId || y.RegionId == value.RegionId)).ToList().AsReadOnly();
                }
            }
            catch (Exception ex)
            {
                Trace.WriteLine(ex.Message);
            }
            return null;
        }


        public void SaveCatalog(Catalog value)
        {
            Trace.WriteLine(value.Title);
            using (CatalogDataAccessor a = new CatalogDataAccessor())
            {
                a.InsertOrUpdate(value);
                a.SaveChanges();
            }
        }


        public List<Size> GetSizes(int count)
        {
            using(SizeDataAccessor a = new SizeDataAccessor())
            {
                return a.CreateQuery().Take(count).ToList();
            }
        }
        
        public void InsertOrUpdateSize(List<Size> entities)
        {
            if (entities == null || entities.Count <= 0)
            {
                throw new ArgumentException("Entities parameter must not be null and must contain at least one object");
            }

            using (SizeDataAccessor a = new SizeDataAccessor())
            {
                a.InsertOrUpdate(entities.ToArray());
                a.SaveChanges();
            }
        }


        public void InsertOrUpdateSizeEntity(Size entity)
        {
            if (entity == null)
            {
                throw new ArgumentException("Entities parameter must not be null");
            }

            using (SizeDataAccessor a = new SizeDataAccessor())
            {
                a.InsertOrUpdate(entity);
                a.SaveChanges();
            }
        }


        public Genre GetItemsByGenre(string title)
        {
            if (string.IsNullOrEmpty(title))
            {
                throw new ArgumentException("Title argument must not be null or empty");
            }

            using (DataAccessor<Genre> a = new DataAccessor<Genre>())
            {
                return a.CreateQuery().Include("Catalogs").Where(x => x.Title.ToUpper() == title.ToUpper()).FirstOrDefault();
            }
        }

        public void UpdateGenre(Genre entity)
        {
            using (DataAccessor<Genre> a = new DataAccessor<Genre>())
            {
                a.InsertOrUpdate(entity);
                a.SaveChanges();
            }        
        }


        public List<Genre> GetAllGenres()
        {
            using (DataAccessor<Genre> a = new DataAccessor<Genre>())
            {
                return a.CreateQuery().Include("Catalogs").ToList();
            }
        }

        public void UpdateGenres(params Genre[] entities)
        {
            using (DataAccessor<Genre> a = new DataAccessor<Genre>())
            {
                a.InsertOrUpdate(entities);
                a.SaveChanges();
            }
        }

        public List<Genre> GetGenres(int count)
        {
            using (DataAccessor<Genre> a = new DataAccessor<Genre>())
            {
                return a.CreateQuery().Take(count).Include("Catalogs").ToList();
            }
        }


        public Catalog GetCatalogById(int Id)
        {
            using (CatalogDataAccessor access = new CatalogDataAccessor())
            {
                return access.GetEntity(x => x.CatalogId == Id);
            }
        }

        public Catalog GetCatalogByIdWithChildren(int Id)
        {
            using (CatalogDataAccessor access = new CatalogDataAccessor())
            {
                return access.GetEntity(x => x.CatalogId == Id, x => x.Genres);
            }
        }
    }
}
